// Textures
Texture2D NormalDepthMap : register(t0);
Texture2D InputImage : register(t1);

SamplerState NormalDepthSampler : register(s0);
SamplerState InputImageSampler : register(s1);

// Constants
cbuffer PerFrameData : register(b0)
{
	float texelWidth;
	float texelHeight;
	float horizontalBlur;
	float padding;
};


struct VSOutput
{
	float4 pos : SV_POSITION;
	float2 tex : TEXCOORD;
};

float4 main(VSOutput input) : SV_Target
{
	float2 texOffset;
	
	float blurRadius = 5.0f;
	float weights[11] = { 0.05f, 0.05f, 0.1f, 0.1f, 0.1f, 0.2f, 0.1f, 0.1f, 0.1f, 0.05f, 0.05f };

	if (horizontalBlur)
	{
		texOffset = float2(texelWidth, 0.0f);
	}
	else
	{
		texOffset = float2(0.0f, texelHeight);
	}

	// The center value always contributes to the sum
	float4 color = weights[5] * InputImage.SampleLevel(InputImageSampler, input.tex, 0.0f);
	float totalWeight = weights[5];
	float4 centerNormalDepth = NormalDepthMap.SampleLevel(NormalDepthSampler, input.tex, 0.0f);

	for (float i = -blurRadius; i <= blurRadius; ++i)
	{
		if (i == 0)
			continue;

		float2 tex = input.tex + i * texOffset;

		float4 neighbourNormalDepth = NormalDepthMap.SampleLevel(NormalDepthSampler, input.tex, 0.0f);

		// avoid sampling across a discontinuity
		if (dot(neighbourNormalDepth.xyz, centerNormalDepth.xyz) >= 0.8f && abs(neighbourNormalDepth.a - centerNormalDepth.a) <= 0.2f)
		{
			float weight = weights[i + blurRadius];

			// Add neighbor pixel to blur
			color += weight * InputImage.SampleLevel(InputImageSampler, tex, 0.0f);

			totalWeight += weight;
		}
	}

	// make total weights sum to 1
	return color / totalWeight;
}
