//Textures
Texture2D diffuseMap : register(t0);
Texture2D lightTexture : register(t1);
SamplerState AnisoSampler : register(s0);

//constants
cbuffer MaterialProperties
{
	float3 specularAlbedo;
	float padding;
};

//input structures
struct PSInput
{
	float4 screenPos : SV_Position;
	float2 texCoord : TEXCOORD0;
};

float4 main(in PSInput input) : SV_Target0
{
	//sample the diffuse map
	float3 diffuseAlbedo = diffuseMap.Sample(AnisoSampler, input.texCoord).rgb;

	//determine the indices for the texture based on the current screen position
	int3 sampleIndices = int3(input.screenPos.xy, 0);

	//sample on the light target
	float4 lighting = lightTexture.Load(sampleIndices);

	//Apply the diffuse and specular albedo to the lighting values
	float3 diffuse = lighting.xyz * diffuseAlbedo;
	float3 specular = lighting.w * specularAlbedo;

	//final out is sum of diffuse and specular
	return float4(diffuse + specular, 1.0f);
}