//constant buffers
cbuffer Transforms
{
	matrix worldMatrix;
	matrix viewMatrix;
	matrix projectionMatrix;
	matrix skinMatrices[6];
	matrix skinNormalMatrices[6];
};

//Input/Output structures
struct VSInput
{
	float4 position : POSITION;
	float2 texCoord : TEXCOORD0;
	float3 normal : NORMAL;
	float4 tangent : TANGENT;
	uint4 bone : BONEIDS;
	float4 weights : BONEWEIGHTS;
};

struct VSOutput
{
	float4 clipPosition : SV_Position;
	float2 texCoord : TEXCOORD0;
	float3 worldNormal : NORMAL;
	float3 worldTangent : TANGENT;
	float3 worldBinormal : BINORMAL;
	float3 worldPosition : POSITION0;
	float viewDepth : POSITION1;
};

//G-Buffer vertex shader (pre-pass lighting)
VSOutput main(in VSInput input)
{
	VSOutput output;

	input.position.w = 1.0f;

	//calculate the output position of the vertex
	output.worldPosition = (mul(input.position, skinMatrices[input.bone.x]) * input.weights.x);
	output.worldPosition += (mul(input.position, skinMatrices[input.bone.y]) * input.weights.y);
	output.worldPosition += (mul(input.position, skinMatrices[input.bone.z]) * input.weights.z);
	output.worldPosition += (mul(input.position, skinMatrices[input.bone.w]) * input.weights.w);

	//calculate the clip space position of the transformed vertices
	output.clipPosition = mul(output.worldPosition, viewMatrix);
	output.clipPosition = mul(output.clipPosition, projectionMatrix);

	//calculate the world space normal vector
	output.worldNormal = (mul(input.normal, (float3x3)skinNormalMatrices[input.bone.x]) * input.weights.x);
	output.worldNormal += (mul(input.normal, (float3x3)skinNormalMatrices[input.bone.y]) * input.weights.y);
	output.worldNormal += (mul(input.normal, (float3x3)skinNormalMatrices[input.bone.z]) * input.weights.z);
	output.worldNormal += (mul(input.normal, (float3x3)skinNormalMatrices[input.bone.w]) * input.weights.w);

	//move position to view space
	float3 viewPosition = mul(input.position, worldMatrix);
	output.viewDepth = mul(viewPosition, viewMatrix).z;

	//reconstruct the rest of the tangent frame
	float3 worldTangent = normalize(mul(input.tangent.xyz, (float3x3)worldMatrix));
	float3 worldBinormal = normalize(cross(worldNormal, worldTangent)) * input.tangent.w;

	output.worldTangent = worldTangent;
	output.worldBinormal = worldBinormal;

	//pass along the texture coordinate
	output.texCoord = input.texCoord;

	return output;
}