//constant buffers
cbuffer Transforms
{
	matrix worldMatrix;
	matrix viewMatrix;
	matrix projectionMatrix;
};

//Input/Output structures
struct VSInput
{
	float4 position : POSITION;
	float2 texCoord : TEXCOORD0;
	float3 normal : NORMAL;
	float4 tangent : TANGENT;
};

struct VSOutput
{
	float4 clipPosition : SV_Position;
	float2 texCoord : TEXCOORD0;
	float3 worldNormal : NORMAL;
	float3 worldTangent : TANGENT;
	float3 worldBinormal : BINORMAL;
	float3 worldPosition : POSITION0;
	float viewDepth : POSITION1;
};

//G-Buffer vertex shader (pre-pass lighting)
VSOutput main(in VSInput input)
{
	VSOutput output;
	
	input.position.w = 1.0f;

	//move position and normals to world space
	output.worldPosition = mul(input.position, worldMatrix);
	float3 worldNormal = normalize(mul(input.normal, (float3x3)worldMatrix));
	output.worldNormal = worldNormal;

	//move position to view space
	float3 viewPosition = mul(input.position, worldMatrix);
	output.viewDepth = mul(viewPosition, viewMatrix).z;

	//reconstruct the rest of the tangent frame
	float3 worldTangent = normalize(mul(input.tangent.xyz, (float3x3)worldMatrix));
	float3 worldBinormal = normalize(cross(worldNormal, worldTangent)) * input.tangent.w;

	output.worldTangent = worldTangent;
	output.worldBinormal = worldBinormal;

	//calculate clip space position
	output.clipPosition = mul(input.position, worldMatrix);
	output.clipPosition = mul(output.clipPosition, viewMatrix);
	output.clipPosition = mul(output.clipPosition, projectionMatrix);

	//pass along the texture coordinate
	output.texCoord = input.texCoord;

	return output;
}