cbuffer MatrixParams
{
	matrix view;
	matrix proj;
};

struct GSInput
{
	float4 pos : SV_POSITION;
	float2 tex : TEXCOORD0;
	float3 normal : NORMAL;
	float3 color : COLOR;
};

struct GSOutput
{
	float3 pos : POSITION;
	float2 tex : TEXCOORD0;
	float3 normal : NORMAL;
	float4 tangent : TANGENT;
};

struct GSRastOutput
{
	float4 pos : SV_POSITION;
	float2 tex : TEXCOORD0;
	float3 normal : NORMAL;
	float4 tangent : TANGENT;
};

float4 CalculateVertexTangent(GSInput p1, GSInput p2, GSInput p3)
{
	float3 tangent;

	float3 vector1 = float3(p2.pos.x - p1.pos.x, p2.pos.y - p1.pos.y, p2.pos.z - p1.pos.z);
	float3 vector2 = float3(p3.pos.x - p1.pos.x, p3.pos.y - p1.pos.y, p3.pos.z - p1.pos.z);

	float2 tuVector = float2(p2.tex.x - p1.tex.x, p3.tex.x - p1.tex.x);
	float2 tvVector = float2(p2.tex.y - p1.tex.y, p3.tex.y - p1.tex.y);

	float den = 1.0f / (tuVector.x * tvVector.y - tuVector.y * tvVector.x);

	tangent.x = (tvVector.y * vector1.x - tvVector.x * vector2.x) * den;
	tangent.y = (tvVector.y * vector1.y - tvVector.x * vector2.y) * den;
	tangent.z = (tvVector.y * vector1.z - tvVector.x * vector2.z) * den;

	tangent = normalize(tangent);

	return float4(tangent, 1.0f);
}

[maxvertexcount(6)]
void main(
	triangle GSInput input[3], 
	inout PointStream< GSOutput > output2,
	inout PointStream< GSRastOutput > output
)
{
	GSRastOutput vertex;
	GSOutput streamVertex;

	// rasterized vertex
	vertex.pos = mul(input[0].pos, view);
	vertex.pos = mul(vertex.pos, proj);
	vertex.tex = input[0].tex;
	vertex.normal = input[0].normal;
	vertex.tangent = CalculateVertexTangent(input[0], input[1], input[2]);

	output.Append(vertex);

	// stream out vertex
	streamVertex.pos = input[0].pos.xyz;
	streamVertex.tex = vertex.tex * 32.0f;
	streamVertex.normal = vertex.normal;
	streamVertex.tangent = vertex.tangent;

	output2.Append(streamVertex);

	// rasterized vertex
	vertex.pos = mul(input[1].pos, view);
	vertex.pos = mul(vertex.pos, proj);
	vertex.tex = input[1].tex;
	vertex.normal = input[1].normal;
	vertex.tangent = CalculateVertexTangent(input[1], input[0], input[2]);

	output.Append(vertex);

	// stream out vertex
	streamVertex.pos = input[1].pos.xyz;
	streamVertex.tex = vertex.tex * 32.0f;
	streamVertex.normal = vertex.normal;
	streamVertex.tangent = vertex.tangent;

	output2.Append(streamVertex);

	// rasterized vertex
	vertex.pos = mul(input[2].pos, view);
	vertex.pos = mul(vertex.pos, proj);
	vertex.tex = input[2].tex;
	vertex.normal = input[2].normal;
	vertex.tangent = CalculateVertexTangent(input[2], input[1], input[0]);

	output.Append(vertex);

	// stream out vertex
	streamVertex.pos = input[2].pos.xyz;
	streamVertex.tex = vertex.tex * 32.0f;
	streamVertex.normal = vertex.normal;
	streamVertex.tangent = vertex.tangent;

	output2.Append(streamVertex);

	output.RestartStrip();
	output2.RestartStrip();
}