// Constants
cbuffer Transforms : register(b0)
{
	matrix worldMatrix;
};

cbuffer NoiseTransforms : register(b1)
{
	float3 cameraPos;
	float terrainSize;
}

// I/O Structure
struct VSInput
{
	float3 position : POSITION;
	float2 texCoord : TEXCOORD;
	float3 normal : NORMAL; // Normals are actually generated on the gpu but this makes the input layout easier
	float3 tangent : TANGENT; // Same as above
};

struct VSOutput
{
	float3 position : WORLD_SPACE_CONTROL_POINT_POSITION;
	float2 texCoord : CONTROL_POINT_TEXCOORD;
	float2 noiseCoord : CONTROL_POINT_NORMAL;
};

VSOutput main(in VSInput input)
{
	VSOutput output;

	output.position = mul(float4(input.position, 1.0f), worldMatrix).xyz;
	output.texCoord = input.texCoord;
	output.noiseCoord = input.texCoord + (cameraPos.xz / terrainSize);

	return output;
}