cbuffer Transforms
{
	matrix world;
	matrix view;
	matrix proj;
};

struct GSOutput
{
	float4 pos : SV_POSITION;
	float2 tex : TEXCOORD0;
	float3 normal : NORMAL;
	float4 tangent : TANGENT;
	float3 worldPos : TEXCOORD1;
};

struct GSInput
{
	float4 pos : SV_POSITION;
	float2 tex : TEXCOORD0;
	float3 normal : NORMAL;
};

float4 CalculateVertexTangent(GSInput p1, GSInput p2, GSInput p3)
{
	float3 tangent;
	
	float3 vector1 = float3(p2.pos.x - p1.pos.x, p2.pos.y - p1.pos.y, p2.pos.z - p1.pos.z);
	float3 vector2 = float3(p3.pos.x - p1.pos.x, p3.pos.y - p1.pos.y, p3.pos.z - p1.pos.z);

	float2 tuVector = float2(p2.tex.x - p1.tex.x, p3.tex.x - p1.tex.x);
	float2 tvVector = float2(p2.tex.y - p1.tex.y, p3.tex.y - p1.tex.y);

	float den = 1.0f / (tuVector.x * tvVector.y - tuVector.y * tvVector.x);

	tangent.x = (tvVector.y * vector1.x - tvVector.x * vector2.x) * den;
	tangent.y = (tvVector.y * vector1.y - tvVector.x * vector2.y) * den;
	tangent.z = (tvVector.y * vector1.z - tvVector.x * vector2.z) * den;

	tangent = normalize(tangent);

	return float4(tangent, 1.0f);
}

float3 CalculateVertexNormal(float4 p1, float4 p2, float4 p3)
{
	float3 vector1, vector2;

	vector1 = p2.xyz - p1.xyz;
	vector2 = p3.xyz - p1.xyz;

	float3 normal = cross(vector1, vector2);

	return normalize(normal);
}

[maxvertexcount(3)]
void main(
	triangle GSInput input[3],
	inout TriangleStream< GSOutput > output
)
{
	GSOutput vertex;
	float2 texCoord;

	// precalculate positions
	float4 p1 = mul(float4(input[0].pos.xyz, 1.0f), world);
	float4 p2 = mul(float4(input[1].pos.xyz, 1.0f), world);
	float4 p3 = mul(float4(input[2].pos.xyz, 1.0f), world);

	texCoord = input[0].normal.xy;
	vertex.pos = mul(mul(p1, view), proj);
	vertex.tex = input[0].tex;
	vertex.worldPos = p1.xyz;
	vertex.normal = CalculateVertexNormal(p1, p2, p3);
	vertex.tangent = CalculateVertexTangent(input[0], input[1], input[2]);
	output.Append(vertex);

	texCoord = input[1].normal.xy;
	vertex.pos = mul(mul(p2, view), proj);
	vertex.tex = input[1].tex;
	vertex.worldPos = p2.xyz;
	vertex.normal = CalculateVertexNormal(p1, p2, p3);
	vertex.tangent = CalculateVertexTangent(input[1], input[0], input[2]);
	output.Append(vertex);

	texCoord = input[2].normal.xy;
	vertex.pos = mul(mul(p3, view), proj);
	vertex.tex = input[2].tex;
	vertex.worldPos = p3.xyz;
	vertex.normal = CalculateVertexNormal(p1, p2, p3);
	vertex.tangent = CalculateVertexTangent(input[2], input[1], input[0]);
	output.Append(vertex);

	output.RestartStrip();
}