cbuffer ReflectionFactors
{
	float4 apexColor;
	float4 centerColor;
	float4 cameraPos;
};

struct GSOutput
{
	float4 pos : SV_POSITION;
	float2 tex : TEXCOORD0;
	float3 normal : NORMAL;
	float4 tangent : TANGENT;
	float3 worldPos : TEXCOORD1;
};

float4 main(in GSOutput input) : SV_TARGET
{
	// determine the height on the sky sphere of the normal
	float height = input.normal.y;

	// values range from -1.0f to +1.0f so change to only position
	height = clamp(abs(height), 0, 1);

	// calculate the fresnel factor
	float Rzero = 1.0f;
	float3 viewVec = input.worldPos - cameraPos;
	//viewVec.x = viewVec.y;
	//viewVec.z = viewVec.y;
	float r = (1.2f - 1.0f) / (1.2f + 1.0f);
	float fresnel = max(0.0f, min(1.0f, r + (1.0f - r) * pow(1.0f - dot(input.normal, viewVec), 2)));

	// determine the gradient color by lerping between apex and center
	float4 color = lerp(centerColor, apexColor, height) * float4(0.0f, 0.8f, 1.0f, 1.0f);
	color.w = fresnel;

	return color;
}