struct GridPoint
{
	float height;
	float4 flow;
};

StructuredBuffer<GridPoint> waterState : register(t0);

cbuffer DispatchParams
{
	float4 dispatchSize;
};

struct VSInput
{
	float3 position : POSITION;
	float2 texCoord : TEXCOORD0;
	float3 normal : NORMAL;
	float4 tangent : TANGENT;
};

struct VSOutput
{
	float4 position : SV_POSITION;
	float2 texCoord : TEXCOORD0;
	float3 normal : NORMAL;
};

VSOutput main( in VSInput input )
{
	VSOutput output;

	output.texCoord = input.texCoord;

	// Find where in the water state buffer to look up for this vertex
	int3 coords = int3(floor(input.texCoord.x), floor(input.texCoord.y), 0);
	int textureIndex = coords.x + coords.y * dispatchSize.z;

	// Sample the heightmap at the specified location
	float height = waterState[textureIndex].height;


	output.position = float4(input.position.x, height, input.position.z, 1.0f);
	output.texCoord = input.texCoord;
	output.normal = input.normal;

	return output;
}