// Mesh.h
#ifndef _BASEMESH_H_
#define _BASEMESH_H_

#include <d3d11.h>
#include <directxmath.h>
#include <vector>
#include "texture.h"

using namespace DirectX;

enum RenderType
{
	INDEXED, AUTO
};

class BaseMesh
{
protected:

	struct VertexType
	{
		XMFLOAT3 position;
		XMFLOAT2 texture;
		XMFLOAT3 normal;
		XMFLOAT4 tangent;
	};

	struct AnimVertexType
	{
		XMFLOAT3 position;
		XMFLOAT2 texture;
		XMFLOAT3 normal;
		XMFLOAT4 tangent;
		XMUINT4 bone;
		XMFLOAT4 weights;
	};

	struct TempVertexType
	{
		float x, y, z;
		float tu, tv;
		float nx, ny, nz;
	};

	struct VectorType
	{
		float x, y, z;
	};

public:
	BaseMesh();
	~BaseMesh();

	virtual void SendData(ID3D11DeviceContext*);
	int GetIndexCount();
	ID3D11ShaderResourceView* GetTexture();
	ID3D11ShaderResourceView* GetBumpMap();

	void LoadBumpMap(ID3D11Device*, ID3D11DeviceContext* deviceContext, WCHAR*);

	ID3D11Buffer* GetVertexBuffer();

	RenderType GetRenderType();

protected:
	virtual void InitBuffers(ID3D11Device*) = 0;
	void LoadTexture(ID3D11Device*, ID3D11DeviceContext* deviceContext,  WCHAR*);
	void CalculateModelVectors(VertexType* vertices);
	void CalculateTangent(TempVertexType, TempVertexType, TempVertexType, VectorType&);
	void CalculateNormal(VectorType, VectorType, VectorType&);

	ID3D11Buffer *m_vertexBuffer, *m_indexBuffer;
	int m_vertexCount, m_indexCount;
	Texture* m_Texture;
	Texture* m_BumpMap;
	bool m_boneAnimated;
	RenderType m_renderType = INDEXED;
};

#endif