// D3D.h
#ifndef _D3D_H_
#define _D3D_H_

// Link libaries dxgi.lib, d3d11.lib, d3dx11.lib, d3dx10.lib

#include <Windows.h>
#include <d3d11.h>
#include <DirectXMath.h>
#include <map>

using namespace DirectX;
using std::map;

class D3D
{
public:
	void* operator new(size_t i)
	{
		return _mm_malloc(i, 16);
	}

	void operator delete(void* p)
	{
		_mm_free(p);
	}

	D3D(int screenWidth, int screenHeight, bool vsync, HWND hwnd, bool fullscreen, float screenDepth, float screenNear);
	~D3D();

	void BeginScene(float r, float g, float b, float a);
	void EndScene();

	ID3D11Device* GetDevice();
	ID3D11DeviceContext* GetDeviceContext();
	ID3D11DepthStencilView* GetDepthStencilView();

	void GetProjectionMatrix(XMMATRIX&);
	void GetWorldMatrix(XMMATRIX&);
	void GetOrthoMatrix(XMMATRIX&);

	void GetVideoCardInfo(char*, int&);

	void TurnZBufferOn();
	void TurnZBufferOff();
	void TurnOnAlphaBlending();
	void TurnOffAlphaBlending();
	void TurnOnWireframe();
	void TurnOffWireframe();

	void SetBackBufferRenderTarget();
	void ResetViewport();

	void CreateSwapChain(std::string, UINT, UINT, bool, DXGI_FORMAT, DXGI_MODE_SCANLINE_ORDER, DXGI_MODE_SCALING, UINT, UINT, DXGI_USAGE, UINT, HWND*, bool, DXGI_SWAP_EFFECT, UINT);

	ID3D11DepthStencilState* GetDefaultDepthStencilState();
	ID3D11BlendState* GetDefaultBlendState();
	ID3D11RasterizerState* GetDefaultRasterizerState();

	int GetScreenWidth();
	int GetScreenHeight();
	float GetFOV();
	float GetScreenNear();
	float GetScreenFar();

protected:
	bool m_vsync_enabled;
	int m_videoCardMemory;
	char m_videoCardDescription[128];
	IDXGISwapChain* m_swapChain;
	ID3D11Device* m_device;
	ID3D11DeviceContext* m_deviceContext;
	ID3D11RenderTargetView* m_renderTargetView;
	ID3D11Texture2D* m_depthStencilBuffer;
	ID3D11DepthStencilState* m_depthStencilState;
	ID3D11DepthStencilState* m_depthDisabledStencilState;
	ID3D11DepthStencilView* m_depthStencilView;
	ID3D11RasterizerState* m_rasterState;
	ID3D11RasterizerState* m_rasterStateWF;
	XMMATRIX m_projectionMatrix;
	XMMATRIX m_worldMatrix;
	XMMATRIX m_orthoMatrix;
	ID3D11BlendState* m_alphaEnableBlendingState;
	ID3D11BlendState* m_alphaDisableBlendingState;
	D3D11_VIEWPORT viewport;
	int m_screenWidth, m_screenHeight;
	float m_fov, m_nearClip, m_farClip;
};


ID3D11Buffer* CreateVertexBuffer(ID3D11Device* device, UINT size, bool dynamic, bool streamout, D3D11_SUBRESOURCE_DATA* pData);
ID3D11Buffer* CreateIndexBuffer(ID3D11Device* device, UINT size, bool dynamic, D3D11_SUBRESOURCE_DATA* pData);
ID3D11Buffer* CreateConstantBuffer(ID3D11Device* device, UINT size, bool dynamic, bool CPUupdates, D3D11_SUBRESOURCE_DATA* pData);
ID3D11Buffer* CreateStagingBuffer(ID3D11Device* device, UINT size, D3D11_SUBRESOURCE_DATA* pData);

#endif