// Light class
// Holds data that represents a light source

#include "light.h"

Light::~Light()
{
	if (m_shadowMap)
	{
		delete m_shadowMap;
		m_shadowMap = 0;
	}
}

void Light::GenerateViewMatrix()
{
	XMVECTOR up = XMVectorSet(0.0f, 1.0f, 0.0f, 1.0f);

	//up = XMVectorSetX(up, 0.0f);
	//up = XMVectorSetY(up, 1.0f);
	//up = XMVectorSetZ(up, 0.0f);

	// Create the view matrix from the three vectors.
	m_viewMatrix = XMMatrixLookAtLH(m_position, m_lookAt, up);
}

void Light::GenerateProjectionMatrix(float screenNear, float screenFar)
{
	float fieldOfView, screenAspect;

	// Setup field of view and screen aspect for a square light source.
	fieldOfView = (float)XM_PI / 2.0f;
	screenAspect = 1.0f;

	// Create the projection matrix for the light.
	m_projectionMatrix = XMMatrixPerspectiveFovLH(fieldOfView, screenAspect, screenNear, screenFar);
}

void Light::GenerateShadowMapBuffer(ID3D11Device* device, ID3D11DeviceContext* deviceContext, int width, int height, int mapCount, int sampleCount)
{
	m_shadowMap = new DeferredBuffers(device, deviceContext, width, height, mapCount, sampleCount);
}

void Light::SetDiffuseColour(float red, float green, float blue, float alpha)
{
	m_diffuseColour = XMFLOAT4(red, green, blue, alpha);
}


void Light::SetDirection(float x, float y, float z)
{
	m_direction = XMFLOAT3(x, y, z);
}

void Light::SetPosition(float x, float y, float z)
{
	m_position = XMVectorSet(x, y, z, 1.0f);
}

void Light::SetConstantAttenuation(float constant)
{
	m_constantAttenuation = constant;
}

void Light::SetLinearAttenuation(float linear)
{
	m_linearAttenuation = linear;
}

void Light::SetQuadraticAttenuation(float quadratic)
{
	m_quadraticAttenuation = quadratic;
}

void Light::SetLightType(int type)
{
	m_lightType = type;
}

XMFLOAT4 Light::GetDiffuseColour()
{
	return m_diffuseColour;
}


XMFLOAT3 Light::GetDirection()
{
	return m_direction;
}

XMFLOAT3 Light::GetPosition()
{
	XMFLOAT3 temp(XMVectorGetX(m_position), XMVectorGetY(m_position), XMVectorGetZ(m_position));
	return temp;
}

void Light::SetLookAt(float x, float y, float z)
{
	m_lookAt = XMVectorSet(x, y, z, 1.0f);
}

XMMATRIX Light::GetViewMatrix()
{
	return m_viewMatrix;
}

XMMATRIX Light::GetProjectionMatrix()
{
	return m_projectionMatrix;
}

float Light::GetConstantAttenuation()
{
	return m_constantAttenuation;
}

float Light::GetLinearAttenuation()
{
	return m_linearAttenuation;
}

float Light::GetQuadraticAttenuation()
{
	return m_quadraticAttenuation;
}

float Light::GetLightType()
{
	return m_lightType;
}

DeferredBuffers* Light::GetShadowMap()
{
	return m_shadowMap;
}