#include "RenderWindowMesh.h"

RenderWindowMesh::RenderWindowMesh(ID3D11Device* device, int screenWidth, int screenHeight)
{
	m_screenWidth = screenWidth;
	m_screenHeight = screenHeight;

	InitBuffers(device);
}

RenderWindowMesh::~RenderWindowMesh()
{
	BaseMesh::~BaseMesh();
}

void RenderWindowMesh::InitBuffers(ID3D11Device* device)
{
	float left, right, top, bottom;
	VertexType* vertices;
	unsigned long* indices;
	D3D11_SUBRESOURCE_DATA vertexData, indexData;
	HRESULT result;

	left = (float)((m_screenWidth / 2) * -1);
	right = left + (float)m_screenWidth;
	top = (float)(m_screenHeight / 2);
	bottom = top - (float)m_screenHeight;

	m_vertexCount = 6;
	m_indexCount = m_vertexCount;

	vertices = new VertexType[m_vertexCount];

	//Build the vertex array
	/*
	vertices[0].position = XMFLOAT3(left, bottom, 0.0f);
	vertices[1].position = XMFLOAT3(left, top, 0.0f);
	vertices[2].position = XMFLOAT3(right, top, 0.0f);
	vertices[3].position = XMFLOAT3(left, bottom, 0.0f);
	vertices[4].position = XMFLOAT3(right, top, 0.0f);
	vertices[5].position = XMFLOAT3(right, bottom, 0.0f);
	*/
	vertices[0].position = XMFLOAT3(-1.0f, -1.0f, 0.0f);
	vertices[1].position = XMFLOAT3(-1.0f, 1.0f, 0.0f);
	vertices[2].position = XMFLOAT3(1.0f, 1.0f, 0.0f);
	vertices[3].position = XMFLOAT3(-1.0f, -1.0f, 0.0f);
	vertices[4].position = XMFLOAT3(1.0f, 1.0f, 0.0f);
	vertices[5].position = XMFLOAT3(1.0f, -1.0f, 0.0f);
	
	indices = new unsigned long[m_indexCount];

	for (int i = 0; i < m_indexCount; i++)
	{
		indices[i] = i;
	}

	vertexData.pSysMem = vertices;
	vertexData.SysMemPitch = 0;
	vertexData.SysMemSlicePitch = 0;

	m_vertexBuffer = CreateVertexBuffer(device, sizeof(VertexType) * m_vertexCount, true, false, &vertexData);

	indexData.pSysMem = indices;
	indexData.SysMemPitch = 0;
	indexData.SysMemSlicePitch = 0;

	m_indexBuffer = CreateIndexBuffer(device, sizeof(unsigned long) * m_indexCount, true, &indexData);

	delete[] vertices;
	vertices = nullptr;

	delete[] indices;
	indices = nullptr;
}

void RenderWindowMesh::SendData(ID3D11DeviceContext* deviceContext)
{
	unsigned int stride;
	unsigned int offset;

	stride = sizeof(VertexType);
	offset = 0;

	deviceContext->IASetVertexBuffers(0, 1, &m_vertexBuffer, &stride, &offset);
	deviceContext->IASetIndexBuffer(m_indexBuffer, DXGI_FORMAT_R32_UINT, 0);
	deviceContext->IASetPrimitiveTopology(D3D11_PRIMITIVE_TOPOLOGY_TRIANGLELIST);
}