#include "ResourceContainer.h"

ResourceContainer::ResourceContainer()
{
}

ResourceContainer::~ResourceContainer()
{
	for (int i = 0; i < m_textures.size(); i++)
	{
		delete m_textures[i].m_texture;
		m_textures[i].m_texture = 0;
	}
	m_textures.clear();

	for (int i = 0; i < m_meshes.size(); i++)
	{
		delete m_meshes[i].m_mesh;
		m_meshes[i].m_mesh = 0;
	}
	m_meshes.clear();

	for (int i = 0; i < m_shaders.size(); i++)
	{
		delete m_shaders[i].m_shader;
		m_shaders[i].m_shader = 0;
	}
	m_shaders.clear();

	for (int i = 0; i < m_buffers.size(); i++)
	{
		delete m_buffers[i].m_buffers;
		m_buffers[i].m_buffers = 0;
	}
	m_buffers.clear();

	for (int i = 0; i < m_cameras.size(); i++)
	{
		delete m_cameras[i].m_camera;
		m_cameras[i].m_camera = 0;
	}
	m_cameras.clear();

	for (int i = 0; i < m_lights.size(); i++)
	{
		delete m_lights[i].m_light;
		m_lights[i].m_light = 0;
	}
	m_lights.clear();
}

void ResourceContainer::ClearResources()
{
	for (int i = 0; i < m_textures.size(); i++)
	{
		delete m_textures[i].m_texture;
		m_textures[i].m_texture = 0;
	}
	m_textures.clear();

	for (int i = 0; i < m_meshes.size(); i++)
	{
		delete m_meshes[i].m_mesh;
		m_meshes[i].m_mesh = 0;
	}
	m_meshes.clear();

	for (int i = 0; i < m_shaders.size(); i++)
	{
		delete m_shaders[i].m_shader;
		m_shaders[i].m_shader = 0;
	}
	m_shaders.clear();

	for (int i = 0; i < m_buffers.size(); i++)
	{
		delete m_buffers[i].m_buffers;
		m_buffers[i].m_buffers = 0;
	}
	m_buffers.clear();

	for (int i = 0; i < m_cameras.size(); i++)
	{
		delete m_cameras[i].m_camera;
		m_cameras[i].m_camera = 0;
	}
	m_cameras.clear();

	for (int i = 0; i < m_lights.size(); i++)
	{
		delete m_lights[i].m_light;
		m_lights[i].m_light = 0;
	}
	m_lights.clear();
}

int ResourceContainer::AddTexture(const std::wstring& name, Texture* texture)
{
	for (int i = 0; i < m_textures.size(); i++)
	{
		if (m_textures[i].m_name == name)
			return -1;
	}
	m_textures.push_back(TextureStore(name, texture));
	return (m_textures.size() - 1);
}

int ResourceContainer::AddMesh(const std::wstring& name, BaseMesh* mesh)
{
	for (int i = 0; i < m_meshes.size(); i++)
	{
		if (m_meshes[i].m_name == name)
			return -1;
	}
	m_meshes.push_back(MeshStore(name, mesh));
	return (m_meshes.size() - 1);
}

int ResourceContainer::AddShader(const std::wstring& name, BaseShader* shader)
{
	for (int i = 0; i < m_shaders.size(); i++)
	{
		if (m_shaders[i].m_name == name)
			return -1;
	}
	m_shaders.push_back(ShaderStore(name, shader));
	return (m_shaders.size() - 1);
}

int ResourceContainer::AddBuffer(const std::wstring& name, DeferredBuffers* buffer)
{
	for (int i = 0; i < m_buffers.size(); i++)
	{
		if (m_buffers[i].m_name == name)
			return -1;
	}
	m_buffers.push_back(BufferStore(name, buffer));
	return (m_buffers.size() - 1);
}

int ResourceContainer::AddCamera(const std::wstring& name, Camera* camera)
{
	for (int i = 0; i < m_cameras.size(); i++)
	{
		if (m_cameras[i].m_name == name)
			return -1;
	}
	m_cameras.push_back(CameraStore(name, camera));
	return (m_cameras.size() - 1);
}

int ResourceContainer::AddLight(const std::wstring& name, Light* light)
{
	for (int i = 0; i < m_lights.size(); i++)
	{
		if (m_lights[i].m_name == name)
			return -1;
	}
	m_lights.push_back(LightStore(name, light));
	return (m_lights.size() - 1);
}

int ResourceContainer::AddUAV(const std::wstring& name, UnorderedAccessView* uav)
{
	for (int i = 0; i < m_uavs.size(); ++i)
	{
		if (m_uavs[i].m_name == name)
			return -1;
	}
	m_uavs.push_back(UAVStore(name, uav));
	return (m_uavs.size() - 1);
}

int ResourceContainer::AddFloat(const std::wstring& name, FloatParameter* floatParam)
{
	for (int i = 0; i < m_floats.size(); ++i)
	{
		if (m_floats[i].m_name == name)
			return -1;
	}
	m_floats.push_back(FloatStore(name, floatParam));
	return (m_floats.size() - 1);
}
Texture* ResourceContainer::GetTexture(int textureIndex)
{
	if (textureIndex >= m_textures.size() || textureIndex < 0)
		return NULL;

	return m_textures[textureIndex].m_texture;
}

Texture* ResourceContainer::GetTexture(const std::wstring& name)
{
	for (auto tex : m_textures)
	{
		if (tex.m_name == name)
			return tex.m_texture;
	}
	return NULL;
}

int ResourceContainer::GetTextureIndex(const std::wstring& name)
{
	for (int index = 0; index < m_textures.size(); index++)
	{
		if (m_textures[index].m_name == name)
			return index;
	}
	return -1;
}

BaseMesh* ResourceContainer::GetMesh(int meshIndex)
{
	if (meshIndex >= m_meshes.size() || meshIndex < 0)
		return NULL;
	return m_meshes[meshIndex].m_mesh;
}

BaseMesh* ResourceContainer::GetMesh(const std::wstring& name)
{
	for (auto mesh : m_meshes)
	{
		if (mesh.m_name == name)
			return mesh.m_mesh;
	}
	return NULL;
}

int ResourceContainer::GetMeshIndex(const std::wstring& name)
{
	for (int index = 0; index < m_meshes.size(); index++)
	{
		if (m_meshes[index].m_name == name)
			return index;
	}
	return -1;
}

BaseShader* ResourceContainer::GetShader(int shaderIndex)
{
	if (shaderIndex >= m_shaders.size() || shaderIndex < 0)
		return NULL;
	return m_shaders[shaderIndex].m_shader;
}

BaseShader* ResourceContainer::GetShader(const std::wstring& name)
{
	for (auto shader : m_shaders)
	{
		if (shader.m_name == name)
			return shader.m_shader;
	}
	return NULL;
}

int ResourceContainer::GetShaderIndex(const std::wstring& name)
{
	for (int index = 0; index < m_shaders.size(); index++)
	{
		if (m_shaders[index].m_name == name)
			return index;
	}
	return -1;
}

DeferredBuffers* ResourceContainer::GetBuffers(int bufferIndex)
{
	if (bufferIndex >= m_buffers.size() || bufferIndex < 0)
		return NULL;
	return m_buffers[bufferIndex].m_buffers;
}

DeferredBuffers* ResourceContainer::GetBuffers(const std::wstring& name)
{
	for (auto buffer : m_buffers)
	{
		if (buffer.m_name == name)
			return buffer.m_buffers;
	}
	return NULL;
}

int ResourceContainer::GetBufferIndex(const std::wstring& name)
{
	for (int index = 0; index < m_buffers.size(); index++)
	{
		if (m_buffers[index].m_name == name)
			return index;
	}
	return -1;
}

Camera* ResourceContainer::GetCamera(int cameraIndex)
{
	if (cameraIndex >= m_cameras.size() || cameraIndex < 0)
		return NULL;
	return m_cameras[cameraIndex].m_camera;
}

Camera* ResourceContainer::GetCamera(const std::wstring& name)
{
	for (auto camera : m_cameras)
	{
		if (camera.m_name == name)
			return camera.m_camera;
	}
	return NULL;
}

int ResourceContainer::GetCameraIndex(const std::wstring& name)
{
	for (int index = 0; index < m_cameras.size(); index++)
	{
		if (m_cameras[index].m_name == name)
			return index;
	}
	return -1;
}

Light* ResourceContainer::GetLight(int lightIndex)
{
	if (lightIndex >= m_lights.size() || lightIndex < 0)
		return NULL;
	return m_lights[lightIndex].m_light;
}

Light* ResourceContainer::GetLight(const std::wstring& name)
{
	for (auto light : m_lights)
	{
		if (light.m_name == name)
			return light.m_light;
	}
	return NULL;
}

int ResourceContainer::GetLightIndex(const std::wstring& name)
{
	for (int index = 0; index < m_lights.size(); index++)
	{
		if (m_lights[index].m_name == name)
			return index;
	}
	return -1;
}

UnorderedAccessView* ResourceContainer::GetUAV(int uavIndex)
{
	if (uavIndex >= m_uavs.size() || uavIndex < 0)
		return NULL;
	return m_uavs[uavIndex].m_uav;
}

UnorderedAccessView* ResourceContainer::GetUAV(const std::wstring& name)
{
	for (auto uav : m_uavs)
	{
		if (uav.m_name == name)
			return uav.m_uav;
	}
	return NULL;
}

int ResourceContainer::GetUAVIndex(const std::wstring& name)
{
	for (int i = 0; i < m_uavs.size(); ++i)
	{
		if (m_uavs[i].m_name == name)
			return i;
	}

	return -1;
}

FloatParameter* ResourceContainer::GetFloat(int floatIndex)
{
	if (floatIndex >= m_floats.size() || floatIndex < 0)
		return NULL;
	return m_floats[floatIndex].m_float;
}

FloatParameter* ResourceContainer::GetFloat(const std::wstring& name)
{
	for (auto floatParam : m_floats)
	{
		if (floatParam.m_name == name)
			return floatParam.m_float;
	}
	return NULL;
}

int ResourceContainer::GetFloatIndex(const std::wstring& name)
{
	for (int i = 0; i < m_floats.size(); ++i)
	{
		if (m_floats[i].m_name == name)
			return i;
	}

	return -1;
}