#ifndef _RESOURCECONTAINER_H_
#define _RESOURCECONTAINER_H_

#include "BaseMesh.h"
#include "BaseShader.h"
#include "Texture.h"
#include "DeferredBuffers.h"
#include "UnorderedAccessView.h"
#include "FloatParameter.h"
#include "Camera.h"

class ResourceContainer
{
	static ResourceContainer *m_containerInstance;
public:
	struct TextureStore
	{
		Texture* m_texture;
		std::wstring m_name;
		TextureStore(const std::wstring& name, Texture* texture) { m_name = name; m_texture = texture; }
	};
	struct MeshStore
	{
		BaseMesh* m_mesh;
		std::wstring m_name;
		MeshStore(const std::wstring& name, BaseMesh* mesh) { m_name = name; m_mesh = mesh; }
	};
	struct ShaderStore
	{
		BaseShader* m_shader;
		std::wstring m_name;
		ShaderStore(const std::wstring& name, BaseShader* shader) { m_name = name; m_shader = shader; }
	};
	struct BufferStore
	{
		DeferredBuffers* m_buffers;
		std::wstring m_name;
		BufferStore(const std::wstring& name, DeferredBuffers* buffers) { m_name = name; m_buffers = buffers; }
	};
	struct CameraStore
	{
		Camera* m_camera;
		std::wstring m_name;
		CameraStore(const std::wstring& name, Camera* camera) { m_name = name; m_camera = camera; }
	};
	struct LightStore
	{
		Light* m_light;
		std::wstring m_name;
		LightStore(const std::wstring& name, Light* light) { m_name = name; m_light = light; }
	};
	struct UAVStore
	{
		UnorderedAccessView* m_uav;
		std::wstring m_name;
		UAVStore(const std::wstring& name, UnorderedAccessView* uav) { m_name = name, m_uav = uav; }
	};

	struct FloatStore
	{
		FloatParameter* m_float;
		std::wstring m_name;
		FloatStore(const std::wstring& name, FloatParameter* floatParam) { m_name = name, m_float = floatParam; }
	};
	static ResourceContainer *instance()
	{
		if (!m_containerInstance)
			m_containerInstance = new ResourceContainer;
		return m_containerInstance;
	}

	ResourceContainer();
	~ResourceContainer();

	void ClearResources();

	int AddTexture(const std::wstring& name, Texture* texture);
	int AddMesh(const std::wstring& name, BaseMesh* mesh);
	int AddShader(const std::wstring& name, BaseShader* shader);
	int AddBuffer(const std::wstring& name, DeferredBuffers* buffer);
	int AddCamera(const std::wstring& name, Camera* camera);
	int AddLight(const std::wstring& name, Light* light);
	int AddUAV(const std::wstring& name, UnorderedAccessView* uav);
	int AddFloat(const std::wstring& name, FloatParameter* floatParam);

	Texture* GetTexture(int textureIndex);
	Texture* GetTexture(const std::wstring& name);
	int GetTextureIndex(const std::wstring& name);

	BaseMesh* GetMesh(int meshIndex);
	BaseMesh* GetMesh(const std::wstring& name);
	int GetMeshIndex(const std::wstring& name);

	BaseShader* GetShader(int shaderIndex);
	BaseShader* GetShader(const std::wstring& name);
	int GetShaderIndex(const std::wstring& name);

	DeferredBuffers* GetBuffers(int bufferIndex);
	DeferredBuffers* GetBuffers(const std::wstring& name);
	int GetBufferIndex(const std::wstring& name);

	Camera* GetCamera(int cameraIndex);
	Camera* GetCamera(const std::wstring& name);
	int GetCameraIndex(const std::wstring& name);

	Light* GetLight(int lightIndex);
	Light* GetLight(const std::wstring& name);
	int GetLightIndex(const std::wstring& name);

	UnorderedAccessView* GetUAV(int UAVIndex);
	UnorderedAccessView* GetUAV(const std::wstring& name);
	int GetUAVIndex(const std::wstring& name);

	FloatParameter* GetFloat(int floatIndex);
	FloatParameter* GetFloat(const std::wstring& name);
	int GetFloatIndex(const std::wstring& name);

	inline int GetTextureCount() { return m_textures.size(); }
	inline int GetMeshCount() { return m_meshes.size(); }
	inline int GetShaderCount() { return m_shaders.size(); }
	inline int GetBufferCount() { return m_buffers.size(); }
	inline int GetCameraCount() { return m_cameras.size(); }
	inline int GetLightCount() { return m_lights.size(); }
	inline int GetUAVCount() { return m_uavs.size(); }
	inline int GetFloatCount() { return m_floats.size(); }

private:
	std::vector<TextureStore> m_textures;
	std::vector<MeshStore> m_meshes;
	std::vector<ShaderStore> m_shaders;
	std::vector<BufferStore> m_buffers;
	std::vector<CameraStore> m_cameras;
	std::vector<LightStore> m_lights;
	std::vector<UAVStore> m_uavs;
	std::vector<FloatStore> m_floats;
};

#endif