#include "StreamOutputMesh.h"

StreamOutputMesh::StreamOutputMesh(ID3D11Device* device, ID3D11DeviceContext* deviceContext, int vertexCount)
{
	m_vertexCount = vertexCount;
	m_renderType = AUTO;
	InitBuffers(device);
}

StreamOutputMesh::~StreamOutputMesh()
{
	BaseMesh::~BaseMesh();
}

void StreamOutputMesh::InitBuffers(ID3D11Device* device)
{
	HRESULT result;
	unsigned long *indices;
	D3D11_BUFFER_DESC vertexBufferDesc, indexBufferDesc;
	D3D11_SUBRESOURCE_DATA indexData;

	m_indexCount = m_vertexCount;
	
	indices = new unsigned long[m_indexCount];

	for (int i = 0; i < m_indexCount; ++i)
		indices[i] = i;

	// Set up the description of the static vertex buffer.
	vertexBufferDesc.Usage = D3D11_USAGE_DEFAULT;
	vertexBufferDesc.ByteWidth = sizeof(VertexType)* m_vertexCount;
	vertexBufferDesc.BindFlags = (D3D11_BIND_VERTEX_BUFFER | D3D11_BIND_STREAM_OUTPUT);
	vertexBufferDesc.CPUAccessFlags = 0;
	vertexBufferDesc.MiscFlags = 0;

	// Now create the vertex buffer.
	result = device->CreateBuffer(&vertexBufferDesc, NULL, &m_vertexBuffer);

	// Set up the description of the static index buffer.
	indexBufferDesc.Usage = D3D11_USAGE_DEFAULT;
	indexBufferDesc.ByteWidth = sizeof(unsigned long)* m_indexCount;
	indexBufferDesc.BindFlags = D3D11_BIND_INDEX_BUFFER;
	indexBufferDesc.CPUAccessFlags = 0;
	indexBufferDesc.MiscFlags = 0;
	indexBufferDesc.StructureByteStride = 0;

	// Give the subresource structure a pointer to the index data.
	indexData.pSysMem = indices;
	indexData.SysMemPitch = 0;
	indexData.SysMemSlicePitch = 0;

	// Create the index buffer.
	result = device->CreateBuffer(&indexBufferDesc, &indexData, &m_indexBuffer);

	// Release the arrays now that the vertex and index buffers have been created and loaded.
	delete[] indices;
	indices = 0;
}

void StreamOutputMesh::SendData(ID3D11DeviceContext* deviceContext)
{
	unsigned int stride;
	unsigned int offset;

	// Set vertex buffer stride and offset.
	stride = (m_boneAnimated) ? sizeof(AnimVertexType) : sizeof(VertexType);
	offset = 0;

	// Set the vertex buffer to active in the input assembler so it can be rendered.
	deviceContext->IASetVertexBuffers(0, 1, &m_vertexBuffer, &stride, &offset);

	// Set the index buffer to inactive in the input assembler
	deviceContext->IASetIndexBuffer(NULL, DXGI_FORMAT_R32_UINT, 0);

	// Set the type of primitive that should be rendered from this vertex buffer, in this case triangles.
	deviceContext->IASetPrimitiveTopology(D3D11_PRIMITIVE_TOPOLOGY_TRIANGLELIST);
}