#include "HeightmapGenerationShader.h"
#include "../DXFramework/ResourceContainer.h"

HeightmapGenerationShader::HeightmapGenerationShader(ID3D11Device* device, HWND hwnd) : BaseShader(device, hwnd)
{
	InitShader(L"../res/shaders/heightmap_generation_cs.hlsl");
}

HeightmapGenerationShader::~HeightmapGenerationShader()
{
	// cleanup the buffer
	if (m_heightmapFactorsBuffer)
	{
		m_heightmapFactorsBuffer->Release();
		m_heightmapFactorsBuffer = 0;
	}
}

void HeightmapGenerationShader::InitShader(WCHAR* vsFilename, WCHAR* psFilename)
{
}

void HeightmapGenerationShader::InitShader(WCHAR* csFilename)
{
	loadComputeShader(csFilename);

	// create the heightmap generation factors buffer
	m_heightmapFactorsBuffer = CreateConstantBuffer(m_device, sizeof(HeightmapGenerationFactors), true, true, NULL);
}

void HeightmapGenerationShader::Render(ID3D11DeviceContext* deviceContext, D3D* d3d, void* resourceContainer)
{
	D3D11_MAPPED_SUBRESOURCE mappedResource;
	HeightmapGenerationFactors* heightmapGenerationFactors;

	ResourceContainer* container = reinterpret_cast<ResourceContainer*>(resourceContainer);

	// Set the heightmap factors in the shader
	deviceContext->Map(m_heightmapFactorsBuffer, 0, D3D11_MAP_WRITE_DISCARD, 0, &mappedResource);
	heightmapGenerationFactors = (HeightmapGenerationFactors*)mappedResource.pData;
	heightmapGenerationFactors->octaves = 6.0f;
	heightmapGenerationFactors->amplitude = 0.5f;
	heightmapGenerationFactors->frequency = 3.0f;
	heightmapGenerationFactors->resolution = 512.0f;
	deviceContext->Unmap(m_heightmapFactorsBuffer, 0);
	deviceContext->CSSetConstantBuffers(0, 1, &m_heightmapFactorsBuffer);

	// Bind the heightmap uav to the compute shader
	ID3D11UnorderedAccessView* heightmap = container->GetUAV(m_renderUAVs[0])->GetUAV();
	deviceContext->CSSetUnorderedAccessViews(0, 1, &heightmap, NULL);

	// Set the thread x, y and z thread counts
	m_threadGroupCountX = heightmapGenerationFactors->resolution / 32;
	m_threadGroupCountY = heightmapGenerationFactors->resolution / 32;
	m_threadGroupCountZ = 1;

	BaseShader::Render(deviceContext, d3d, resourceContainer);

	// unbind the heightmap after rendering is completed
	ID3D11UnorderedAccessView* NULL_UAV = NULL;
	deviceContext->CSSetUnorderedAccessViews(0, 1, &NULL_UAV, 0);
}

