#include "HeightmapProcessShader.h"
#include "../DXFramework/ResourceContainer.h"

HeightmapProcessShader::HeightmapProcessShader(ID3D11Device* device, HWND hwnd) : BaseShader(device, hwnd)
{
	InitShader(L"../res/shaders/heightmap_preprocess_cs.hlsl");
}

HeightmapProcessShader::~HeightmapProcessShader()
{
}

void HeightmapProcessShader::InitShader(WCHAR* csFilename)
{
	HRESULT result;
	D3D11_SAMPLER_DESC samplerDesc;

	loadComputeShader(csFilename);

	// Create a texture sampler state description.
	samplerDesc.Filter = D3D11_FILTER_MIN_MAG_MIP_LINEAR;
	samplerDesc.AddressU = D3D11_TEXTURE_ADDRESS_CLAMP;
	samplerDesc.AddressV = D3D11_TEXTURE_ADDRESS_CLAMP;
	samplerDesc.AddressW = D3D11_TEXTURE_ADDRESS_CLAMP;
	samplerDesc.MipLODBias = 0.0f;
	samplerDesc.MaxAnisotropy = 16;
	samplerDesc.ComparisonFunc = D3D11_COMPARISON_ALWAYS;
	samplerDesc.BorderColor[0] = 0;
	samplerDesc.BorderColor[1] = 0;
	samplerDesc.BorderColor[2] = 0;
	samplerDesc.BorderColor[3] = 0;
	samplerDesc.MinLOD = 0;
	samplerDesc.MaxLOD = D3D11_FLOAT32_MAX;

	// Create the texture sampler state.
	result = m_device->CreateSamplerState(&samplerDesc, &m_sampleState);
	if (FAILED(result))
	{
		printf("Sample State creation failed.");
	}
}

void HeightmapProcessShader::InitShader(WCHAR* vsFilename, WCHAR* psFilename)
{
}

void HeightmapProcessShader::Render(ID3D11DeviceContext* deviceContext, D3D* d3d, void* resourceContainer)
{
	ResourceContainer* container = reinterpret_cast<ResourceContainer*>(resourceContainer);

	// Set the sampler in the Compute shader
	deviceContext->CSSetSamplers(0, 1, &m_sampleState);

	// Bind the unordered access view to the compute shader
	ID3D11UnorderedAccessView* lodLookup = container->GetUAV(m_renderUAVs[0])->GetUAV();
	deviceContext->CSSetUnorderedAccessViews(0, 1, &lodLookup, NULL);

	// Set the heightmap in the compute shader
	ID3D11ShaderResourceView* heightmap = container->GetUAV(m_renderUAVs[1])->GetSRV();
	deviceContext->CSSetShaderResources(0, 1, &heightmap);

	// Set the thread x, y and z thread counts
	m_threadGroupCountX = 512 / 16;
	m_threadGroupCountY = 512 / 16;
	m_threadGroupCountZ = 1;

	BaseShader::Render(deviceContext, d3d, resourceContainer);

	ID3D11UnorderedAccessView* NULL_UAV = NULL;
	deviceContext->CSSetUnorderedAccessViews(0, 1, &NULL_UAV, 0);
}