#include "PrePassMaskShader.h"
#include "../DXFramework/ResourceContainer.h"

PrePassMaskShader::PrePassMaskShader(ID3D11Device* device, HWND hwnd) : BaseShader(device, hwnd)
{
	m_deferredShader = true;
	InitShader(L"../res/shaders/prepass_mask_vs.hlsl", L"../res/shaders/prepass_mask_ps.hlsl");
}

PrePassMaskShader::~PrePassMaskShader()
{
	// Release the sampler state.
	if (m_sampleState)
	{
		m_sampleState->Release();
		m_sampleState = 0;
	}

	// Release the layout.
	if (m_layout)
	{
		m_layout->Release();
		m_layout = 0;
	}

	BaseShader::~BaseShader();
}

void PrePassMaskShader::InitShader(WCHAR* vsFilename, WCHAR* psFilename)
{
	loadVertexShader(vsFilename);

	loadPixelShader(psFilename);
}

void PrePassMaskShader::Render(ID3D11DeviceContext* deviceContext, D3D* d3d, void* resourceContainer)
{
	ResourceContainer* container = reinterpret_cast<ResourceContainer*>(resourceContainer);

	container->GetMesh(m_renderObjects[0]->GetMeshIndex(0))->SendData(deviceContext);
	m_indexCount = container->GetMesh(m_renderObjects[0]->GetMeshIndex(0))->GetIndexCount();

	ID3D11ShaderResourceView* gBufferTex = container->GetBuffers(L"G_BUFFER")->GetShaderResourceView(0);

	deviceContext->PSSetShaderResources(0, 1, &gBufferTex);

	deviceContext->PSSetSamplers(0, 1, &m_sampleState);

	BaseShader::Render(deviceContext, d3d, resourceContainer);
}
