#ifndef _TERRAINSHADER_H_
#define _TERRAINSHADER_H_

#include "../DXFramework/BaseShader.h"
#include "../DXFramework/Light.h"

class TerrainShader : public BaseShader
{
private:
	// Buffer Structs
	struct VSMatrixBufferType
	{
		XMMATRIX world;
	};

	struct VSNoiseBufferType
	{
		XMFLOAT3 cameraPos;
		float terrainSize;
	};

	struct HSCameraBufferType
	{
		XMFLOAT3 cameraPos;
		float padding;
	};

	struct HSLODParamBufferType
	{
		XMFLOAT4 minMaxDistance;
		XMFLOAT4 minMaxLOD;
	};

	struct DSMatrixBufferType
	{
		XMMATRIX invTposeWorld;
	};

	struct DSSampleParamBufferType
	{
		XMFLOAT4 heightMapDimensions;
	};

	struct GSMatrixBufferType
	{
		XMMATRIX view;
		XMMATRIX proj;
	};

	struct PSMaterialBufferType
	{
		XMFLOAT3 specularAlbedo;
		float specularPower;
	};

	struct PSCameraBufferType
	{
		XMFLOAT3 cameraPos;
		float padding;
	};

public:
	TerrainShader(ID3D11Device* device, HWND hwnd);
	~TerrainShader();

	void Render(ID3D11DeviceContext* deviceContext, D3D* d3d, void* resourceContainer);

private:
	void InitShader(WCHAR*, WCHAR*);
	void InitShader(WCHAR*, WCHAR*, WCHAR*, WCHAR*, WCHAR*);

protected:
	ID3D11Buffer* m_vsMatrixBuffer;
	ID3D11Buffer* m_vsNoiseBuffer;
	ID3D11Buffer* m_hsCameraBuffer;
	ID3D11Buffer* m_hsLODParamBuffer;
	ID3D11Buffer* m_hsSampleParamBuffer;
	ID3D11Buffer* m_dsSampleParamBuffer;
	ID3D11Buffer* m_gsMatrixBuffer;
	ID3D11Buffer* m_psLightBuffer;
	ID3D11Buffer* m_psMaterialBuffer;
	ID3D11Buffer* m_psCameraBuffer;


};

#endif