#include "WaterMesh.h"

WaterMesh::WaterMesh(ID3D11Device* device, int sizeX, int sizeZ)
{
	m_sizeX = sizeX;
	m_sizeZ = sizeZ;

	InitBuffers(device);
}

WaterMesh::~WaterMesh()
{
}

void WaterMesh::InitBuffers(ID3D11Device* device)
{
	HRESULT result;
	VertexType* vertices;
	D3D11_BUFFER_DESC vertexBufferDesc, indexBufferDesc;
	D3D11_SUBRESOURCE_DATA vertexData, indexData;
	unsigned long* indices;
	int vertex, index;

	m_vertexCount = m_sizeX * m_sizeZ * 6;
	m_indexCount = m_vertexCount;

	vertices = new VertexType[m_vertexCount];

	// Set the locations and texture coordinates first. Also pass the real texture coordinates as the normals
	for (int x = 0; x < m_sizeX * 2; x++)
	{
		for (int z = 0; z < m_sizeZ * 2; z++)
		{
			int posX = x / 2;
			int posZ = z / 2;

			int vertex = (posX * m_sizeZ * 6) + (posZ * 6);

			// move the x and z positions to be zero centred
			float xPos = (float)posX - ((float)m_sizeX / 2.0f);
			float zPos = (float)posZ - ((float)m_sizeZ / 2.0f);

			// Bottom left
			vertices[vertex].position = XMFLOAT3(xPos, 0.0f, zPos);
			vertices[vertex].texture = XMFLOAT2((float)posX, (float)posZ);
			vertices[vertex].normal = XMFLOAT3((float)posX / (float)m_sizeX, (float)posZ / (float)m_sizeZ, 0.0f);
			vertices[vertex].normal = XMFLOAT3(0.0f, 0.0f, 0.0f);

			// Bottom right
			vertices[vertex + 1].position = XMFLOAT3(xPos + 1.0f, 0.0f, zPos);
			vertices[vertex + 1].texture = XMFLOAT2((float)posX + 1.0f, (float)posZ);
			vertices[vertex + 1].normal = XMFLOAT3(((float)posX + 1.0f) / (float)m_sizeX, (float)posZ / (float)m_sizeZ, 0.0f);
			vertices[vertex + 1].normal = XMFLOAT3(1.0f, 0.0f, 0.0f);

			// Top left
			vertices[vertex + 2].position = XMFLOAT3(xPos, 0.0f, zPos + 1.0f);
			vertices[vertex + 2].texture = XMFLOAT2((float)posX, (float)posZ + 1.0f);
			vertices[vertex + 2].normal = XMFLOAT3((float)posX / (float)m_sizeX, ((float)posZ + 1.0f) / (float)m_sizeZ, 0.0f);
			vertices[vertex + 2].normal = XMFLOAT3(0.0f, 1.0f, 0.0f);

			// Bottom right
			vertices[vertex + 3].position = XMFLOAT3(xPos + 1.0f, 0.0f, zPos);
			vertices[vertex + 3].texture = XMFLOAT2((float)posX + 1.0f, (float)posZ);
			vertices[vertex + 3].normal = XMFLOAT3(((float)posX + 1.0f)/ (float)m_sizeX, (float)posZ / (float)m_sizeZ, 0.0f);
			vertices[vertex + 3].normal = XMFLOAT3(1.0f, 0.0f, 0.0f);

			// Top right
			vertices[vertex + 4].position = XMFLOAT3(xPos + 1.0f, 0.0f, zPos + 1.0f);
			vertices[vertex + 4].texture = XMFLOAT2((float)posX + 1.0f, (float)posZ + 1.0f);
			vertices[vertex + 4].normal = XMFLOAT3(((float)posX + 1.0f) / (float)m_sizeX, ((float)posZ + 1.0f) / (float)m_sizeZ, 0.0f);
			vertices[vertex + 4].normal = XMFLOAT3(1.0f, 1.0f, 0.0f);

			// Top left
			vertices[vertex + 5].position = XMFLOAT3(xPos, 0.0f, zPos + 1.0f);
			vertices[vertex + 5].texture = XMFLOAT2((float)posX, (float)posZ + 1.0f);
			vertices[vertex + 5].normal = XMFLOAT3((float)posX / (float)m_sizeX, ((float)posZ + 1.0f)/ (float)m_sizeZ, 0.0f);
			vertices[vertex + 5].normal = XMFLOAT3(0.0f, 1.0f, 0.0f);
		}
	}

	// Set the basic index data
	indices = new unsigned long[m_indexCount];

	for (int i = 0; i < m_indexCount; ++i)
		indices[i] = i;

	// Set up the description of the static vertex buffer.
	vertexBufferDesc.Usage = D3D11_USAGE_DEFAULT;
	vertexBufferDesc.ByteWidth = sizeof(VertexType)* m_vertexCount;
	vertexBufferDesc.BindFlags = D3D11_BIND_VERTEX_BUFFER;
	vertexBufferDesc.CPUAccessFlags = 0;
	vertexBufferDesc.MiscFlags = 0;
	vertexBufferDesc.StructureByteStride = 0;

	// Give the subresource structure a pointer to the vertex data.
	vertexData.pSysMem = vertices;
	vertexData.SysMemPitch = 0;
	vertexData.SysMemSlicePitch = 0;

	// Now create the vertex buffer.
	result = device->CreateBuffer(&vertexBufferDesc, &vertexData, &m_vertexBuffer);

	// Set up the description of the static index buffer.
	indexBufferDesc.Usage = D3D11_USAGE_DEFAULT;
	indexBufferDesc.ByteWidth = sizeof(unsigned long)* m_indexCount;
	indexBufferDesc.BindFlags = D3D11_BIND_INDEX_BUFFER;
	indexBufferDesc.CPUAccessFlags = 0;
	indexBufferDesc.MiscFlags = 0;
	indexBufferDesc.StructureByteStride = 0;

	// Give the subresource structure a pointer to the index data.
	indexData.pSysMem = indices;
	indexData.SysMemPitch = 0;
	indexData.SysMemSlicePitch = 0;

	// Create the index buffer.
	result = device->CreateBuffer(&indexBufferDesc, &indexData, &m_indexBuffer);

	// Release the arrays now that the buffers have been created and loaded.
	delete[] vertices;
	vertices = 0;

	delete[] indices;
	indices = 0;
}
