#ifndef _WATERSIMULATIONSHADER_H_
#define _WATERSIMULATIONSHADER_H_

#include "../DXFramework/BaseShader.h"

struct FluidColumnState
{
	float height;
	XMFLOAT4 flow;
};

class WaterSimulationShader : public BaseShader
{
protected:
	struct DispatchParams
	{
		XMFLOAT4 dispatchSize;
	};

	struct TimeParams
	{
		XMFLOAT4 timeFactors;
		XMFLOAT4 dispatchSize;
	};

	struct SimulationParams
	{
		float timeStep;
		float pipeArea;
		float gravitation;
		float pipeLength;
		float fluidDensity;
		float columnArea;
		float dampingFactor;
		float padding;
	};

	struct ReflectionParams
	{
		XMFLOAT4 apexColor;
		XMFLOAT4 centerColor;
		XMFLOAT4 cameraPos;
	};

public:
	WaterSimulationShader(ID3D11Device* device, HWND hwnd);
	~WaterSimulationShader();

	void Render(ID3D11DeviceContext* deviceContext, D3D* d3d, void* resourceContainer);

protected:
	void InitShader(WCHAR*, WCHAR*, WCHAR*, WCHAR*);
	void InitShader(WCHAR*, WCHAR*);

protected:
	ID3D11Buffer* m_dispatchParamsBuffer;
	ID3D11Buffer* m_timeParamsBuffer;
	ID3D11Buffer* m_simulationParamsBuffer;
	ID3D11Buffer* m_reflectionParamsBuffer;

	bool m_uavSwitch = false;
};

#endif